/* computes rete for H 21 cm from upper to lower excitation by atomic hydrogen 
 * from A.C. Allison & A. Dalgarno, 1969, ApJ 158, 423 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cddefines.h"
#include "h21cm.h"

/* computes rete for H 21 cm from upper to lower excitation by atomic hydrogen 
 * from A.C. Allison & A. Dalgarno, 1969, ApJ 158, 423 */
double h21_t_ge_20( double temp )
{
	double y;
	double x1;
	/* data go up to 1,000K must not go above this */
	temp = MIN2( 1000.,temp );
	x1 =1.0/sqrt(temp);
	y =-21.70880995483007-13.76259674006133*x1;
	return(exp(y));
}

/* this branch for T < 20K, data go down to 1 K */
double h21_t_lt_20( double temp )
{
	double y;
	double x1;

	/* must not go below 1K */
	temp = MAX2(1., temp );
	x1 =temp*log(temp);
	y =9.720710314268267E-08+6.325515312006680E-08*x1;
	return(y*y);
}

double H21cm( double temp )
{
	double hold ;
	if( temp >= 20. )
	{
		hold = h21_t_ge_20( temp );
	}
	else
	{
		hold = h21_t_lt_20( temp );
	}

	return hold;
}
